@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
 /* ui.c - main menus and callbacks; utility functions
 
-   Copyright (C) 1998 - 2003 Free Software Foundation
+   Copyright (C) 1998 - 2004 Free Software Foundation
 
    GHex is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -23,16 +23,19 @@
 
 #include <config.h>
 #include <gnome.h>
+#include <string.h>
 
 #include <libgnomeprint/gnome-print.h>
 #include <libgnomeprintui/gnome-print-dialog.h>
 #include <libgnomeprintui/gnome-print-job-preview.h>
 
 #include "ghex.h"
+#include "ghex-window.h"
 
 static void ghex_print(GtkHex *gh, gboolean preview);
-static gboolean ghex_print_run_dialog(GHexPrintJobInfo *pji);
+static void ghex_print_run_dialog(GHexPrintJobInfo *pji);
 static void ghex_print_preview_real(GHexPrintJobInfo *pji);
+static void ghex_print_document_real (GHexPrintJobInfo *pji, gboolean preview);
 
 /* callbacks to nullify widget pointer after a delete event */
 static void about_destroy_cb(GtkObject *, GtkWidget **);
@@ -88,6 +91,7 @@ BonoboUIVerb ghex_verbs [] = {
 	BONOBO_UI_VERB ("EditCopy", copy_cb),
 	BONOBO_UI_VERB ("EditPaste", paste_cb),
 	BONOBO_UI_VERB ("Find", find_cb),
+	BONOBO_UI_VERB ("AdvancedFind", advanced_find_cb),
 	BONOBO_UI_VERB ("Replace", replace_cb),
 	BONOBO_UI_VERB ("GoToByte", jump_cb),
 	BONOBO_UI_VERB ("AddView", add_view_cb),
@@ -134,41 +138,17 @@ cancel_cb(GtkWidget *w, GtkWidget *me)
 }
 
 gint
-delete_event_cb(GtkWidget *w, GdkEventAny *e)
+delete_event_cb(GtkWidget *w, GdkEventAny *e, GtkWindow *win)
 {
 	gtk_widget_hide(w);
 	
 	return TRUE;
 }
 
-void
-file_sel_ok_cb(GtkWidget *w, gboolean *resp)
-{
-	*resp = TRUE;
-	gtk_main_quit();
-}
-
-void
-file_sel_cancel_cb(GtkWidget *w, gboolean *resp)
-{
-	*resp = FALSE;
-	gtk_main_quit();
-}
-
 gint
-file_sel_delete_event_cb(GtkWidget *w, GdkEventAny *e, gboolean *resp)
+ask_user(GtkMessageDialog *message_box)
 {
-	*resp = FALSE;
-	gtk_main_quit();
-	return TRUE;
-}
-
-gint
-ask_user(GnomeMessageBox *message_box)
-{
-	gtk_window_set_modal(GTK_WINDOW(message_box), TRUE);	
-
-	return gnome_dialog_run_and_close(GNOME_DIALOG(message_box));
+	return gtk_dialog_run(GTK_DIALOG(message_box));
 }
 
 GtkWidget *
@@ -241,7 +221,7 @@ about_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
 
 	if(!about) {
 		about = gnome_about_new ( _("GHex, a binary file editor"), VERSION,
-								  "(C) 1998 - 2002 Jaka Mocnik",
+								  "(C) 1998 - 2004 Jaka Mocnik",
 								  _("Released under the terms of GNU Public License"),
 								  authors, NULL, NULL, NULL);
 		g_signal_connect(G_OBJECT(about), "destroy",
@@ -282,7 +262,6 @@ void
 paste_cb(BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
 {
 	GHexWindow *win = GHEX_WINDOW(user_data);
-	HexDocument *doc;
 
 	if(win->gh)
 		gtk_hex_paste_clipboard(win->gh);
@@ -292,7 +271,6 @@ void
 copy_cb(BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
 {
 	GHexWindow *win = GHEX_WINDOW(user_data);
-	HexDocument *doc;
 
 	if(win->gh)
 		gtk_hex_copy_clipboard(win->gh);
@@ -302,7 +280,6 @@ void
 cut_cb(BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
 {
 	GHexWindow *win = GHEX_WINDOW(user_data);
-	HexDocument *doc;
 
 	if(win->gh)
 		gtk_hex_cut_clipboard(win->gh);
@@ -311,7 +288,7 @@ cut_cb(BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
 void
 quit_app_cb(BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
 {
-	const GList *win_node, *doc_node;
+	const GList *doc_node;
 	GHexWindow *win;
 	HexDocument *doc;
 
@@ -365,44 +342,34 @@ static void
 open_cb(BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
 {
 	GHexWindow *win;
-	HexDocument *doc;
 	GtkWidget *file_sel;
-	gboolean resp;
+	GtkResponseType resp;
 
 	win = GHEX_WINDOW(user_data);
 
-	file_sel = gtk_file_selection_new(NULL);
-
-	gtk_window_set_title(GTK_WINDOW(file_sel), _("Select a file to open"));
-	
-	gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (file_sel)->ok_button),
-						"clicked", GTK_SIGNAL_FUNC(file_sel_ok_cb),
-					    &resp);
-	gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (file_sel)->cancel_button),
-						"clicked", GTK_SIGNAL_FUNC(file_sel_cancel_cb),
-						&resp);
-	gtk_signal_connect (GTK_OBJECT (file_sel),
-						"delete-event", GTK_SIGNAL_FUNC(file_sel_delete_event_cb),
-						&resp);
-
+	file_sel = gtk_file_chooser_dialog_new(_("Select a file to open"),
+										   GTK_WINDOW(win),
+										   GTK_FILE_CHOOSER_ACTION_OPEN,
+										   GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+										   GTK_STOCK_OPEN, GTK_RESPONSE_OK,
+										   NULL);
 	gtk_window_set_modal (GTK_WINDOW(file_sel), TRUE);
-	gtk_window_position (GTK_WINDOW (file_sel), GTK_WIN_POS_MOUSE);
+	gtk_window_set_position (GTK_WINDOW (file_sel), GTK_WIN_POS_MOUSE);
 	gtk_widget_show (file_sel);
 
-	gtk_main();
+	resp = gtk_dialog_run(GTK_DIALOG(file_sel));
 
-	if(resp) {
-		HexDocument *new_doc;
+	if(resp == GTK_RESPONSE_OK) {
 		gchar *flash;
 
 		if(GHEX_WINDOW(win)->gh != NULL) {
-			win = GHEX_WINDOW(ghex_window_new_from_file(gtk_file_selection_get_filename (GTK_FILE_SELECTION (file_sel))));
+			win = GHEX_WINDOW(ghex_window_new_from_file(gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (file_sel))));
 			if(win != NULL)
 				gtk_widget_show(GTK_WIDGET(win));
 		}
 		else {
 			if(!ghex_window_load(GHEX_WINDOW(win),
-								 gtk_file_selection_get_filename (GTK_FILE_SELECTION (file_sel))))
+								 gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (file_sel))))
 				win = NULL;
 		}
 
@@ -422,7 +389,7 @@ open_cb(BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
 			display_error_dialog (ghex_window_get_active(), _("Can not open file!"));
 	}
 
-	gtk_widget_destroy(GTK_WIDGET(file_sel));
+	gtk_widget_destroy(file_sel);
 }
 
 static void
@@ -470,7 +437,7 @@ export_html_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbnam
 	GHexWindow *win = GHEX_WINDOW(user_data);
 	HexDocument *doc;
 	GtkWidget *file_sel;
-	gboolean resp;
+	GtkResponseType resp;
 
 	if(win->gh)
 		doc = win->gh->document;
@@ -480,27 +447,23 @@ export_html_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbnam
 	if(doc == NULL)
 		return;
 
-	file_sel = gtk_file_selection_new(NULL);
-
-	gtk_file_selection_set_filename(GTK_FILE_SELECTION(file_sel), doc->file_name);
-	gtk_window_set_title(GTK_WINDOW(file_sel), _("Select path and file name for the HTML source"));	
-	gtk_signal_connect(GTK_OBJECT (GTK_FILE_SELECTION (file_sel)->ok_button),
-						"clicked", GTK_SIGNAL_FUNC(file_sel_ok_cb),
-						&resp);
-	gtk_signal_connect(GTK_OBJECT (GTK_FILE_SELECTION (file_sel)->cancel_button),
-						"clicked", GTK_SIGNAL_FUNC(file_sel_cancel_cb),
-						&resp);
-	gtk_signal_connect (GTK_OBJECT (file_sel),
-						"delete-event", GTK_SIGNAL_FUNC(file_sel_delete_event_cb),
-						&resp);
+	file_sel = gtk_file_chooser_dialog_new(_("Select path and file name for the HTML source"),
+										   GTK_WINDOW(win),
+										   GTK_FILE_CHOOSER_ACTION_SAVE,
+										   GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+										   GTK_STOCK_SAVE, GTK_RESPONSE_OK,
+										   NULL);
+#if 0
+	gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(file_sel), doc->file_name);
+#endif
 	gtk_window_set_modal(GTK_WINDOW(file_sel), TRUE);
+	gtk_window_set_position(GTK_WINDOW (file_sel), GTK_WIN_POS_MOUSE);
 	gtk_widget_show(file_sel);
-	gtk_window_position(GTK_WINDOW (file_sel), GTK_WIN_POS_MOUSE);
 
-	gtk_main();
+	resp = gtk_dialog_run(GTK_DIALOG(file_sel));
 
-	if(resp) {
-		gchar *html_path = g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(file_sel)));
+	if(resp == GTK_RESPONSE_OK) {
+		gchar *html_path = g_strdup(gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(file_sel)));
 		gchar *sep, *base_name, *check_path;
 		GtkHex *view = win->gh;
 
@@ -526,7 +489,7 @@ export_html_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbnam
 		check_path = g_strdup_printf("%s/%s.html", html_path, base_name);
 		if(access(check_path, F_OK) == 0) {
 			gint reply;
-			GnomeMessageBox *mbox;
+			GtkWidget *mbox;
 
 			if(access(check_path, W_OK) != 0) {
 				display_error_dialog(win, _("You don't have the permission to write to the selected path.\n"));
@@ -535,15 +498,17 @@ export_html_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbnam
 				return;
 			}
 
-			mbox = GNOME_MESSAGE_BOX(gnome_message_box_new(_("Saving to HTML will overwrite some files.\n"
-															 "Do you want to proceed?"),
-														   GNOME_MESSAGE_BOX_QUESTION,
-														   GNOME_STOCK_BUTTON_YES,
-														   GNOME_STOCK_BUTTON_NO,
-														   NULL));
-			gnome_dialog_set_default(GNOME_DIALOG(mbox), 2);
-			reply = ask_user(mbox);
-			if(reply != 0) {
+			mbox = gtk_message_dialog_new(GTK_WINDOW(win),
+										  GTK_DIALOG_MODAL|
+										  GTK_DIALOG_DESTROY_WITH_PARENT,
+										  GTK_MESSAGE_QUESTION,
+										  GTK_BUTTONS_YES_NO,
+										  _("Saving to HTML will overwrite some files.\n"
+											"Do you want to proceed?"));
+			gtk_dialog_set_default_response(GTK_DIALOG(mbox), GTK_RESPONSE_NO);
+			reply = ask_user(GTK_MESSAGE_DIALOG(mbox));
+			gtk_widget_destroy(mbox);
+			if(reply != GTK_RESPONSE_YES) {
 				g_free(html_path);
 				g_free(check_path);
 				return;
@@ -594,6 +559,7 @@ close_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
 			gtk_widget_destroy(GTK_WIDGET(other_win));
 	}
 
+
 	/* If we have created the converter window disable the 
 	 * "Get cursor value" button
 	 */
@@ -609,8 +575,8 @@ close_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
     else
         gtk_widget_destroy(GTK_WIDGET(win));	
 
-	/* this implicitly destroys all views including this one */
-	g_object_unref(G_OBJECT(doc));
+    /* this implicitly destroys all views including this one */
+    g_object_unref(G_OBJECT(doc));
 }
 
 void
@@ -656,7 +622,7 @@ void prefs_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname
 		gtk_window_set_transient_for(GTK_WINDOW(prefs_ui->pbox),
 									 GTK_WINDOW(ghex_window_get_active()));
 	if(!GTK_WIDGET_VISIBLE(prefs_ui->pbox)) {
-		gtk_window_position (GTK_WINDOW(prefs_ui->pbox), GTK_WIN_POS_MOUSE);
+		gtk_window_set_position (GTK_WINDOW(prefs_ui->pbox), GTK_WIN_POS_MOUSE);
 		gtk_widget_show(GTK_WIDGET(prefs_ui->pbox));
 	}
 	raise_and_focus_widget(GTK_WIDGET(prefs_ui->pbox));
@@ -666,10 +632,9 @@ void prefs_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname
 static void
 revert_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
 {
-	static gchar msg[MESSAGE_LEN + 1];
 	GHexWindow *win;
    	HexDocument *doc;
-	GnomeMessageBox *mbox;
+	GtkWidget *mbox;
 	gint reply;
 	
 	win = GHEX_WINDOW(user_data);
@@ -682,18 +647,18 @@ revert_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
 		return;
 
 	if(doc->changed) {
-		g_snprintf(msg, MESSAGE_LEN,
-				   _("Really revert file %s?"),
-				   doc->path_end);
-		mbox = GNOME_MESSAGE_BOX(gnome_message_box_new(msg,
-													   GNOME_MESSAGE_BOX_QUESTION,
-													   GNOME_STOCK_BUTTON_YES,
-													   GNOME_STOCK_BUTTON_NO,
-													   NULL));
-		gnome_dialog_set_default(GNOME_DIALOG(mbox), 2);
-		reply = ask_user(mbox);
+		mbox = gtk_message_dialog_new(GTK_WINDOW(win),
+									  GTK_DIALOG_MODAL|
+									  GTK_DIALOG_DESTROY_WITH_PARENT,
+									  GTK_MESSAGE_QUESTION,
+									  GTK_BUTTONS_YES_NO,
+									  _("Really revert file %s?"),
+									  doc->path_end);
+		gtk_dialog_set_default_response(GTK_DIALOG(mbox), GTK_RESPONSE_NO);
+
+		reply = ask_user(GTK_MESSAGE_DIALOG(mbox));
 		
-		if(reply == 0) {
+		if(reply == GTK_RESPONSE_YES) {
 			gchar *flash;
 			gchar *gtk_file_name;
 
@@ -707,6 +672,8 @@ revert_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
 			g_free(gtk_file_name);
 			g_free(flash);
 		}
+
+		gtk_widget_destroy (mbox);
 	}
 }
 
@@ -722,7 +689,6 @@ ghex_print(GtkHex *gh, gboolean preview)
 {
 	GHexPrintJobInfo *pji;
 	HexDocument *doc;
-	gboolean cancel = FALSE;
 
 	doc = gh->document;
 
@@ -734,30 +700,100 @@ ghex_print(GtkHex *gh, gboolean preview)
 	pji->preview = preview;
 
 	if (!pji->preview)
-		cancel = ghex_print_run_dialog(pji);
+		ghex_print_run_dialog(pji);
 	else
 		pji->config = gnome_print_config_default();
 
-	/* Cancel clicked */
-	if (cancel) {
+	if (pji->preview) {
+		ghex_print_document_real(pji, preview);
 		ghex_print_job_info_destroy(pji);
-		return;
 	}
+}
+
+static void
+ghex_print_progress(gint page, gint ofpages, gpointer data)
+{
+	GtkProgressBar *progress_bar = GTK_PROGRESS_BAR(data);
+	gchar *progress_str;
+
+	gtk_progress_bar_set_fraction(progress_bar,
+								  (gdouble)page/(gdouble)ofpages);
+	progress_str = g_strdup_printf("%d/%d", page, ofpages);
+	gtk_progress_bar_set_text(progress_bar, progress_str);	
+	g_free(progress_str);
+	while(gtk_events_pending())
+		gtk_main_iteration();
+}
+
+static gboolean
+ignore_cb(GtkWidget *w, GdkEventAny *e, gpointer user_data)
+{
+	return TRUE;
+}
+
+static void
+ghex_print_document_real(GHexPrintJobInfo *pji, gboolean preview)
+{
+	GtkWidget *progress_dialog, *progress_bar;
 
 	g_return_if_fail (pji->config != NULL);
 
 	pji->master = gnome_print_job_new (pji->config);
+
 	g_return_if_fail (pji->master != NULL);
 
-	ghex_print_update_page_size_and_margins (doc, pji);
-	ghex_print_job_execute(pji);
+	ghex_print_update_page_size_and_margins (pji->doc, pji);
 
-	if (pji->preview)
+	progress_dialog = gtk_dialog_new();
+	gtk_window_set_resizable(GTK_WINDOW(progress_dialog), FALSE);
+	gtk_window_set_modal(GTK_WINDOW(progress_dialog), TRUE);
+	g_signal_connect(G_OBJECT(progress_dialog), "delete-event",
+					 G_CALLBACK(ignore_cb), NULL);
+	gtk_window_set_title(GTK_WINDOW(progress_dialog),
+						 _("Printing file..."));
+	progress_bar = gtk_progress_bar_new();
+	gtk_widget_show(progress_bar);
+	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(progress_dialog)->vbox),
+					  progress_bar);
+	gtk_widget_show(progress_dialog);
+
+	ghex_print_job_execute(pji, ghex_print_progress, progress_bar);
+
+	gtk_widget_destroy(progress_dialog);
+
+	if(pji->preview)
 		ghex_print_preview_real(pji);
 	else
 		gnome_print_job_print(pji->master);
+}
 
-	ghex_print_job_info_destroy(pji);
+static void
+ghex_print_dialog_response (GtkWidget *dialog, int response,
+							GHexPrintJobInfo *pji)
+{
+	pji->preview = FALSE;
+ 
+	switch (response) {
+	case GNOME_PRINT_DIALOG_RESPONSE_PRINT:
+		break;
+	case GNOME_PRINT_DIALOG_RESPONSE_PREVIEW:
+		pji->preview = TRUE;
+		break;
+	default:
+		gtk_widget_destroy(dialog);
+		ghex_print_job_info_destroy (pji);
+		return;
+	}
+
+	pji->range = gnome_print_dialog_get_range_page
+		(GNOME_PRINT_DIALOG(dialog), &pji->page_first, &pji->page_last);
+	
+	ghex_print_document_real (pji, pji->preview);
+	
+	if (response == GNOME_PRINT_DIALOG_RESPONSE_PRINT) {
+		ghex_print_job_info_destroy (pji);
+		gtk_widget_destroy(dialog);
+	}
 }
 
 /**
@@ -768,50 +804,35 @@ ghex_print(GtkHex *gh, gboolean preview)
  *
  * Runs the GHex print dialog.
  **/
-static gboolean
+static void
 ghex_print_run_dialog(GHexPrintJobInfo *pji)
 {
 	GtkWidget *dialog;
-	gint res;
 
 	dialog = gnome_print_dialog_new(pji->master,
 			     (const char *) _("Print Hex Document"),
 			     GNOME_PRINT_DIALOG_RANGE);
 
+	if (pji->config == NULL) {
+		pji->config = gnome_print_dialog_get_config(
+						GNOME_PRINT_DIALOG(dialog));
+	}
+
+	ghex_print_update_page_size_and_margins (pji->doc, pji);
+
 	gnome_print_dialog_construct_range_page((GnomePrintDialog *)dialog,
 			GNOME_PRINT_RANGE_ALL | GNOME_PRINT_RANGE_RANGE,
 			1, pji->pages, "A", _("Pages"));
 
-	res = gtk_dialog_run(GTK_DIALOG(dialog));
+	if(ghex_window_get_active() != NULL)
+		gtk_window_set_transient_for(GTK_WINDOW(dialog),
+					     GTK_WINDOW(ghex_window_get_active()));
 
-	switch (res) {
-		case GNOME_PRINT_DIALOG_RESPONSE_PRINT:
-			break;
-		case GNOME_PRINT_DIALOG_RESPONSE_PREVIEW:
-			pji->preview = TRUE;
-			break;
-		case -1:
-			return TRUE;
-		default:
-			gtk_widget_destroy(dialog);
-			return TRUE;
-	};
+	g_signal_connect (dialog, "response",
+			  G_CALLBACK (ghex_print_dialog_response),
+			  pji);
 
-	g_return_val_if_fail (pji->config == NULL, TRUE);
-	pji->config = gnome_print_dialog_get_config(
-			GNOME_PRINT_DIALOG(dialog));
-
-#if 0
-	if (pji->printer && !pji->preview)
-		gnome_print_job_set_printer(pji->master, pji->printer);
-#endif
-
-	pji->range = gnome_print_dialog_get_range_page(
-			GNOME_PRINT_DIALOG(dialog),
-			&pji->page_first, &pji->page_last);
-
-	gtk_widget_destroy (dialog);
-	return FALSE;
+	gtk_widget_show (dialog);
 }
 
 /**
